﻿' add System.DirectoryServices to References
Imports System.DirectoryServices
Imports System.Data.SqlClient

Module Main

    Private LocalUsers_DDL As String = "(Server varchar(50), " _
        & "Name varchar(50), " _
        & "Full_Name varchar(100), " _
        & "Description varchar(100), " _
        & "Login_Script varchar(150), " _
        & "Acct_Locked bit, " _
        & "Acct_Disabled bit, " _
        & "Password_Reset bit, " _
        & "Acct_Expires datetime, " _
        & "Password_Age int, " _
        & "Groups varchar(255), " _
        & "DateMarker datetime, " _
        & "PRIMARY KEY (Server, Name))"

    Private LocalGroups_DDL As String = "(Server varchar(50), " _
        & "GroupName varchar(50), " _
        & "Name varchar(50), " _
        & "Type varchar(50), " _
        & "FullName varchar(100), " _
        & "Description varchar(100), " _
        & "DateMarker datetime, " _
        & "PRIMARY KEY (Server, GroupName, Name))"

    Sub Main()
        Dim con As New SqlConnection
        Dim cmd As New SqlCommand
        Dim cb As SqlCommandBuilder
        Dim da As New SqlDataAdapter
        Dim dt_users As New DataTable
        Dim dt_groups As New DataTable
        Dim dr As DataRow
        Dim el As New EventLog
        Dim dir, dirs, user, grp As DirectoryEntry
        Dim entry, list, temp As Object
        Dim entries As IEnumerable
        Dim svr, svr_list(), groups(10), buf As String
        Dim i, luser_cnt, lgrp_cnt As Integer
        Dim net As New GetNetList
        Dim msg As String = ""

        Console.WriteLine("Local_Accts")
        Console.WriteLine("Start: " & Now())

        Console.WriteLine("Opening the database")
        ' Open the database
        con.ConnectionString = My.Settings.ConnectionString
        Try
            con.Open()
            cmd.Connection = con
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Yikes! Can't open the database" & vbCr & ex.Message)
            el.Source = "Local_Accts"
            el.WriteEntry("Yikes! Can't open the database" & vbCr & ex.Message, EventLogEntryType.Error)
            Exit Sub
        End Try

        ' Create the tables if they don't already exist
        Try
            Dim dt As New DataTable
            Dim got_users, got_groups As Boolean

            da.SelectCommand = cmd
            cmd.CommandText = "SELECT * FROM sysobjects Where xType= 'U'"
            da.Fill(dt)

            For Each dr In dt.Rows
                If dr("name").ToString.ToLower = "localgroups" Then
                    got_groups = True
                End If
                If dr("name").ToString.ToLower = "localusers" Then
                    got_users = True
                End If
            Next

            If Not got_users Then
                cmd.CommandText = "Create Table [LocalUsers] " & LocalUsers_DDL
                cmd.ExecuteNonQuery()
            End If
            If Not got_groups Then
                cmd.CommandText = "Create Table [LocalGroups] " & LocalGroups_DDL
                cmd.ExecuteNonQuery()
            End If
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Yikes! Can't create the tables" & vbCr & ex.Message)
            el.Source = "Local_Accts"
            el.WriteEntry("Yikes! Can't create the tables" & vbCr & ex.Message, EventLogEntryType.Error)
            Exit Sub
        End Try

        ' Empty the existing tables
        Try
            cmd.CommandText = "delete from LocalUsers"
            cmd.ExecuteScalar()
            cmd.CommandText = "delete from LocalGroups"
            cmd.ExecuteScalar()
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Yikes! Can't empty the tables" & vbCr & ex.Message)
            el.Source = "Local_Accts"
            el.WriteEntry("Yikes! Can't empty the tables" & vbCr & ex.Message, EventLogEntryType.Error)
            Exit Sub
        End Try

        ' fill the datatables
        Try
            da.SelectCommand = cmd
            cmd.CommandText = "Select * from LocalUsers"
            da.Fill(dt_users)
            cmd.CommandText = "Select * from LocalGroups"
            da.Fill(dt_groups)
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Yikes! Can't read the tables" & vbCr & ex.Message)
            el.Source = "Local_Accts"
            el.WriteEntry("Yikes! Can't read the tables" & vbCr & ex.Message, EventLogEntryType.Error)
            Exit Sub
        End Try

        Console.WriteLine("Getting Server lists")
        svr_list = net.GetPCOUList(My.Settings.Domain, My.Settings.OU, "", True)

        ' Collect LocalUser and LocalGroup data on all of our Servers
        Console.WriteLine("Getting info on Local Users")
        cmd.CommandText = "Select * from LocalUsers"
        cb = New SqlCommandBuilder(da)

        For Each svr In svr_list
            Try
                dirs = New DirectoryEntry("WinNT://" & My.Settings.Domain.Split(".")(0) & "/" & svr)
                For Each dir In dirs.Children

                    ' start with the LocalUsers
                    If dir.SchemaClassName = "User" Then
                        Console.Write(".")
                        dr = dt_users.NewRow
                        dr("Server") = Left(svr, 50)
                        dr("Name") = Left(dir.Name, 50)
                        dr("Full_Name") = Left(dir.Properties("Fullname").Value, 100)
                        dr("Description") = Left(dir.Properties("Description").Value, 100)
                        dr("Login_Script") = Left(dir.Properties("LoginScript").Value, 150)
                        i = dir.Properties("UserFlags").Value
                        dr("Acct_Disabled") = CBool(i And 2)
                        dr("Acct_Locked") = CBool(i And 16)
                        dr("Password_Reset") = dir.Properties("PasswordExpired").Value
                        temp = dir.Properties("AccountExpirationDate").Value
                        If IsNothing(temp) Then
                            dr("Acct_Expires") = CDate("01/01/80")
                        Else
                            dr("Acct_Expires") = CDate(temp)
                        End If
                        dr("Password_Age") = dir.Properties("PasswordAge").Value / (60 * 60 * 24)
                        ' roll up group memberships into a single line
                        list = dir.Invoke("Groups")
                        entries = CType(list, IEnumerable)
                        i = 0
                        ReDim groups(10)
                        For Each entry In entries
                            grp = New DirectoryEntry(entry)
                            groups(i) = grp.Name
                            i += 1
                            If UBound(groups) < i Then
                                ReDim Preserve groups(i + 9)
                            End If
                        Next
                        ReDim Preserve groups(i - 1)
                        Array.Sort(groups)
                        buf = ""
                        For i = 0 To UBound(groups)
                            If i <> 0 Then
                                buf = buf & ", "
                            End If
                            buf = buf & groups(i)
                        Next
                        dr("Groups") = Left(buf, 255)
                        dr("DateMarker") = Now
                        dt_users.Rows.Add(dr)
                        luser_cnt += 1

                        ' Let's keep the memory footprint down... 
                        If dt_users.Rows.Count > My.Settings.Record_Buffer Then
                            Try
                                Console.Write("*")
                                da.Update(dt_users)
                                dt_users.Clear()
                            Catch ex As Exception
                                Console.WriteLine("Yikes! Can't update the LocalUsers table" & vbCr & ex.Message)
                                el.Source = "Local_Accts"
                                el.WriteEntry("Yikes! Can't update the LocalUsers table" & vbCr & ex.Message, EventLogEntryType.Error)
                                Exit Sub
                            End Try
                        End If
                    End If
                Next
            Catch ex As Exception
                msg &= "Hummm.... Can't get user info from " & svr & ", " & ex.Message & vbCr
                Console.Write("!")
            End Try
        Next
        If dt_users.Rows.Count > 0 Then
            Try
                Console.Write("*")
                da.Update(dt_users)
                dt_users.Clear()
            Catch ex As Exception
                Environment.ExitCode = Err.Number
                Console.WriteLine("Yikes! Can't update the LocalUsers table" & vbCr & ex.Message)
                el.Source = "Local_Accts"
                el.WriteEntry("Yikes! Can't update the LocalUsers table" & vbCr & ex.Message, EventLogEntryType.Error)
                Exit Sub
            End Try
        End If

        ' and now for the Local Groups
        Console.WriteLine("")
        Console.WriteLine("Getting info on Local Groups")

        cmd.CommandText = "Select * from LocalGroups"
        cb = New SqlCommandBuilder(da)

        For Each svr In svr_list
            dirs = New DirectoryEntry("WinNT://" & My.Settings.Domain.Split(".")(0) & "/" & svr)
            Try
                For Each dir In dirs.Children

                    If dir.SchemaClassName = "Group" Then
                        list = dir.Invoke("Members")
                        entries = CType(list, IEnumerable)
                        For Each entry In entries
                            user = New DirectoryEntry(entry)
                            buf = user.Name.ToString
                            If Not buf.EndsWith("$") Then
                                ' too many dots! let's do every 10th one
                                If lgrp_cnt Mod 10 = 0 Then
                                    Console.Write(".")
                                End If
                                dr = dt_groups.NewRow
                                dr("Server") = Left(svr, 50)
                                dr("GroupName") = Left(dir.Name, 50)
                                If buf.StartsWith("S-1-") Then
                                    dr("Name") = Left(buf, 50)
                                Else
                                    dr("Name") = Left(user.Parent.Name & "\" & buf, 50)
                                End If
                                dr("Type") = user.SchemaClassName
                                If user.SchemaClassName = "User" And Not buf.StartsWith("S-1-") Then
                                    ' there's a bug that affects an empty Fullname or Description
                                    ' that causes an exception
                                    Try
                                        If Not IsNothing(user.Properties("Fullname")) Then
                                            dr("FullName") = Left(user.Properties("Fullname").Value, 100)
                                        End If
                                        If Not IsNothing(user.Properties("Description")) Then
                                            dr("Description") = Left(user.Properties("Description").Value, 100)
                                        End If
                                    Catch
                                    End Try
                                End If
                                dr("DateMarker") = Now
                                dt_groups.Rows.Add(dr)
                                lgrp_cnt += 1

                                ' Let's keep the memory footprint down... 
                                If dt_groups.Rows.Count > My.Settings.Record_Buffer Then
                                    Try
                                        Console.Write("*")
                                        da.Update(dt_groups)
                                        dt_groups.Clear()
                                    Catch ex As Exception
                                        Console.WriteLine("Yikes! Can't update the LocalGroups table" & vbCr & ex.Message)
                                        el.Source = "Local_Accts"
                                        el.WriteEntry("Yikes! Can't update the LocalGroups table" & vbCr & ex.Message, EventLogEntryType.Error)
                                        Exit Sub
                                    End Try
                                End If
                            End If
                        Next
                    End If
                Next
            Catch ex As Exception
                msg &= "Hummm.... Can't get user info from " & svr & ", " & ex.Message & vbCr
                Console.Write("!")
            End Try
        Next

        If dt_groups.Rows.Count > 0 Then
            Try
                Console.Write("*")
                da.Update(dt_groups)
                dt_groups.Clear()
            Catch ex As Exception
                Environment.ExitCode = Err.Number
                Console.WriteLine("Yikes! Can't update the LocalGroups table" & vbCr & ex.Message)
                el.Source = "Local_Accts"
                el.WriteEntry("Yikes! Can't update the LocalGroups table" & vbCr & ex.Message, EventLogEntryType.Error)
                Exit Sub
            End Try
        End If

        buf = "LocalUser=" & luser_cnt & vbCrLf & "LocalGroups=" & lgrp_cnt
        el.Source = "Local_Accts"
        If msg <> "" Then
            el.WriteEntry("Local_Acct completed with warnings" & vbCr & buf & vbCr & vbCr & msg, EventLogEntryType.Warning)
        Else
            el.WriteEntry("Local_Acct completed successfully" & vbCr & buf, EventLogEntryType.Information)
        End If

        Console.WriteLine("")
        Console.WriteLine(buf)
        Console.WriteLine("End: " & Now())
        System.Threading.Thread.Sleep(5000)
    End Sub

End Module
